#!/bin/bash
#
# d64_masscopier.sh
#
# written on 30.01.2021  by GI-Joe
#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# This script facilitates the mass copying of floppy disks from a 1541/1571-diskdrive
# to a Linux machine with a XUM-adapter. The XUM-driver, opencbm and nibtools must be
# installed on the system.
# This script will create 1 file in ${DESTINATIONDIR} for each diskside with
# d64copy (normal datacopy):  ${FILEPREFIX}${DISKCOUNTER}${SIDEA|SIDEB}.d64
# +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# usage:
# d64_masscopier.sh [devicenumber] [destinationdir] [destination-file-prefix] [startnumber-diskcounter]
#
# Parameter 1 : devicenumber (8|9|10|11)
# Parameter 2 : destination directory where all D64 files are copied (the directory is created if it does not exist)
# Parameter 3 : prefix of the file name
# Parameter 4 : start number of the disk counter
#
#
#=================================================================================================================================================================
#=================================================================================================================================================================
#==== d64copy-options - just change it at will ===================================================================================================================
#=================================================================================================================================================================
XUMADAPTER="-@xum1541:0"	#-@, --adapter=plugin:bus	-> tell OpenCBM which backend plugin and bus to use
STARTTRACK="-s1"			#-s, --start-track=TRACK	-> set start track
ENDTRACK="-e35"				#-e, --end-track=TRACK		-> set end track (start <= end <= 42/70)
TRANSFERMODE="-ts2"			#-t, --transfer=serial2		-> set transfermode; valid modes: auto (default) | original (slowest) | serial1 or s1 | serial2 or s2 | parallel (fastest)
WARPMODE="-w"				#-w, --warp					-> enable warp mode; this is not possible if TRANSFER is set to `original' This is the default if transfer is not `original'.
RETRYCOUNT="-r10"			#-r, --retry-count=COUNT	-> set retry count
BAMONLY=""					#-b, --bam-only				-> BAM-only copy; 	only allocated blocks are copied; for extended tracks (36-40), SpeedDOS BAM format is assumed. Use with caution.
BAMSAVE=""					#-B, --bam-save				-> save BAM-only copy; this is like the `-b' option but copies always the entire directory track.
ERRORMAP=""					# THIS WILL GROWING THE IMAGE-SIZE (if not -Enever will be set) !!!   -E, --error-map=WHEN		-> control whether the error map is appended. possible values for WHEN are (abbreviations available): always | on_errors (default) | never
TWOSIDED=""					#-2, --two-sided			-> two-sided disk transfer (.d71): Requires 1571. Warp mode is not available for .d71 images.
NOPROGRESS=""				#-n, --no-progress			-> do not display progress information
#
#
#================================================================================================================================================================
#================================================================================================================================================================
#==== variables / constants  - just change it at will ===========================================================================================================
#================================================================================================================================================================
TERMINAL="/usr/bin/xfce4-terminal"	# if you use another terminal, you should edit it here ;)
SIDEA="S1"					# this text will be added in fileprefix for diskside A
SIDEB="S2"					# this text will be added in fileprefix for diskside B
EXTENDED_TRACK_TEST="YES"	# YES|NO - tests each disc before copying whether the tracks 36-41 are readable and if so, these tracks are copied too.
TRACKTESTFILE="/tmp/tracktest.nib"		# temporary filename of the extra-tracks-test - the suffix MUST be ".nib" !!
TRACKTESTLOGFILE="/tmp/tracktest.log"	# temporary filename of the extra-tracks-test - logfile
PARAMETERNUMBER="$#"	# don't change it please !
FILEPREFIX="$3"			# don't change it please !
DEVICENUMBER=`echo "$1" | sed -e 's/^0*//'`		# delete the leading zeros
DESTINATIONDIR=`echo "$2" | sed {"s/\/$//g"}`	# delete the last slash if exist
DISKCOUNTERSTART=`echo 10#$4 | sed -e 's/^0*//'` # delete the leading zeros
if [ "${TWOSIDED}" = "-2" ] || [ "${TWOSIDED}" = "--two-sided" ]
	then
		FILESUFFIX="d71"			# don't change it please !
		EXTENDED_TRACK_TEST="NO"	# don't change it please !
	else
		FILESUFFIX="d64"		# don't change it please !
		TESTSTARTTRACK="-S36" 	# option for nibread (test whether the extra tracks are readable, -Sxx is the starttrack of extratrack-testing)
		TESTENDTRACK="-E41"		# option for nibread (test whether the extra tracks are readable, -Exx is the endtrack of extratrack-testing)
fi
#
#================================================================================================================================================================
#================================================================================================================================================================
#==== functions =================================================================================================================================================
#================================================================================================================================================================
function PrintUsageText {
	echo ""
	echo "at least 1 parameter is missing or incorrect !"
	echo "usage:"
	echo './d64_masscopier.sh [devicenumber] [destinationdir] [destination-file-prefix] [startnumber-diskcounter]'
	echo 'Parameter 1 : devicenumber (8|9|10|11)'
	echo 'Parameter 2 : destination directory where all D64 files are copied (directory is created if does not exist)'
	echo 'Parameter 3 : prefix of the destination filename'
	echo 'Parameter 4 : initial start number of the disk counter'
	echo ""
	echo ""
	exit 1
}
#
function SetExtendedTrackParameters {				# test whether the extra tracks are readable and re-set the option for the endtrack
	[ -f ${TRACKTESTFILE} ] && rm ${TRACKTESTFILE}
	cbmctrl reset
	if cbmctrl detect | grep 1571 >/dev/null 2>&1
		then
			nibread -D$DEVICENUMBER -s $TESTSTARTTRACK $TESTENDTRACK ${TRACKTESTFILE} | tee ${TRACKTESTLOGFILE} # 2>>${TRACKTESTLOGFILE}
		else
			nibread -D$DEVICENUMBER $TESTSTARTTRACK $TESTENDTRACK ${TRACKTESTFILE} | tee ${TRACKTESTLOGFILE} # 2>>${TRACKTESTLOGFILE}
	fi
	sleep 1
	if grep '41.0' ${TRACKTESTLOGFILE} | grep '\[Unformatted Track\]' || grep '41.0' ${TRACKTESTLOGFILE} | grep 'NOSYNC!' || grep '41.0' ${TRACKTESTLOGFILE} | grep '\[NDOS\]' || grep '41.0' ${TRACKTESTLOGFILE} | grep '\[Killer Track\]'
		then
			ENDTRACK="-e40"
		elif grep '41.0' ${TRACKTESTLOGFILE} >/dev/null 2>&1 ; then
			ENDTRACK="-e41" && return
	fi
	if grep '40.0' ${TRACKTESTLOGFILE} | grep '\[Unformatted Track\]' || grep '40.0' ${TRACKTESTLOGFILE} | grep 'NOSYNC!' || grep '40.0' ${TRACKTESTLOGFILE} | grep '\[NDOS\]' || grep '40.0' ${TRACKTESTLOGFILE} | grep '\[Killer Track\]'
		then
			ENDTRACK="-e39"
		elif grep '40.0' ${TRACKTESTLOGFILE} >/dev/null 2>&1 ; then
			ENDTRACK="-e40" && return
	fi
	if grep '39.0' ${TRACKTESTLOGFILE} | grep '\[Unformatted Track\]' || grep '39.0' ${TRACKTESTLOGFILE} | grep 'NOSYNC!' || grep '39.0' ${TRACKTESTLOGFILE} | grep '\[NDOS\]' || grep '39.0' ${TRACKTESTLOGFILE} | grep '\[Killer Track\]'
		then
			ENDTRACK="-e38"
		elif grep '39.0' ${TRACKTESTLOGFILE} >/dev/null 2>&1 ; then
			ENDTRACK="-e39" && return
	fi
	if grep '38.0' ${TRACKTESTLOGFILE} | grep '\[Unformatted Track\]' || grep '38.0' ${TRACKTESTLOGFILE} | grep 'NOSYNC!' || grep '38.0' ${TRACKTESTLOGFILE} | grep '\[NDOS\]' || grep '38.0' ${TRACKTESTLOGFILE} | grep '\[Killer Track\]'
		then
			ENDTRACK="-e37"
		elif grep '38.0' ${TRACKTESTLOGFILE} >/dev/null 2>&1 ; then
			ENDTRACK="-e38" && return
	fi
	if grep '37.0' ${TRACKTESTLOGFILE} | grep '\[Unformatted Track\]' || grep '37.0' ${TRACKTESTLOGFILE} | grep 'NOSYNC!' || grep '37.0' ${TRACKTESTLOGFILE} | grep '\[NDOS\]' || grep '37.0' ${TRACKTESTLOGFILE} | grep '\[Killer Track\]'
		then
			ENDTRACK="-e36"
		elif grep '37.0' ${TRACKTESTLOGFILE} >/dev/null 2>&1 ; then
			ENDTRACK="-e37" && return
	fi
	if grep '36.0' ${TRACKTESTLOGFILE} | grep '\[Unformatted Track\]' || grep '36.0' ${TRACKTESTLOGFILE} | grep 'NOSYNC!' || grep '36.0' ${TRACKTESTLOGFILE} | grep '\[NDOS\]' || grep '36.0' ${TRACKTESTLOGFILE} | grep '\[Killer Track\]'
		then
			ENDTRACK="-e35"
		elif grep '36.0' ${TRACKTESTLOGFILE} >/dev/null 2>&1 ; then
			ENDTRACK="-e36" && return
	fi
}
#
function PrepareForCopySideA {
	[ "$EXTENDED_TRACK_TEST" = "YES" ] && echo -e '\033[1;37mtesting for extra tracks ...\033[0m' && SetExtendedTrackParameters
	[ -f ${TRACKTESTLOGFILE} ] && rm ${TRACKTESTLOGFILE}
	[ -f ${DESTINATIONDIR}/${FILEPREFIX}_${DISKCOUNTERSTRING}-${SIDEA}.${FILESUFFIX} ] && rm ${DESTINATIONDIR}/${FILEPREFIX}_${DISKCOUNTERSTRING}-${SIDEA}.${FILESUFFIX}
	echo "d64copy $XUMADAPTER $DRIVETYPE $STARTTRACK $ENDTRACK $TRANSFERMODE $WARPMODE $RETRYCOUNT $BAMONLY $BAMSAVE $ERRORMAP $TWOSIDED $NOPROGRESS $DEVICENUMBER ${DESTINATIONDIR}/${FILEPREFIX}_${DISKCOUNTERSTRING}-${SIDEA}.${FILESUFFIX}"
	cbmctrl reset
}
#
function PrepareForCopySideB {
	[ "$EXTENDED_TRACK_TEST" = "YES" ] && echo -e '\033[1;37mtesting for extra tracks ...\033[0m' && SetExtendedTrackParameters
	[ -f ${TRACKTESTLOGFILE} ] && rm ${TRACKTESTLOGFILE}
	[ -f ${DESTINATIONDIR}/${FILEPREFIX}_${DISKCOUNTERSTRING}-${SIDEB}.${FILESUFFIX} ] && rm ${DESTINATIONDIR}/${FILEPREFIX}_${DISKCOUNTERSTRING}-${SIDEB}.${FILESUFFIX}
	echo "d64copy $XUMADAPTER $DRIVETYPE $STARTTRACK $ENDTRACK $TRANSFERMODE $WARPMODE $RETRYCOUNT $BAMONLY $BAMSAVE $ERRORMAP $TWOSIDED $NOPROGRESS $DEVICENUMBER ${DESTINATIONDIR}/${FILEPREFIX}_${DISKCOUNTERSTRING}-${SIDEB}.${FILESUFFIX}"
	cbmctrl reset
}
#
#================================================================================================================================================================
#================================================================================================================================================================
#================================================================================================================================================================
#==== Scriptcode ================================================================================================================================================
#================================================================================================================================================================
[ ! -d ${DESTINATIONDIR} ] && mkdir -p ${DESTINATIONDIR}
#
if [ $PARAMETERNUMBER -eq 4 ] # check whether the number of commandline-parameters is correct after script start
	then
		sleep 0
	else
		echo "ERROR: PARAMETERNUMBER is bad !"
		PrintUsageText
fi
#
shopt -s extglob
if [[ -n "$DEVICENUMBER" && -z "${DEVICENUMBER##?([0-9])}" ]] # is parameter 1 an integer ?
	then
		typeset -i DEVICENUMBER
	else
		echo "ERROR: parameter 1 is not an integer"
		PrintUsageText
fi
#
if [[ $DEVICENUMBER -ge 8 && $DEVICENUMBER -le 11 ]] # is the value of parameter 1 between 8 and 11 ?
	then
		typeset -i DEVICENUMBER
	else
		echo "ERROR: parameter 1 is not between 8 and 11"
		PrintUsageText
fi
#
if [[ "$DISKCOUNTERSTART" =~ ^[0-9]+$ ]] # is parameter 4 an integer ?
	then
		echo "ERROR: parameter 4 is not an integer"
		PrintUsageText
	else
		typeset -i DISKCOUNTERSTART
		DISKCOUNTERSTART=$DISKCOUNTERSTART+0
fi
#
if cbmctrl detect >/dev/null 2>&1 	# is the xum1541-adapter connected ?
	then
		sleep 0
	else 
		echo "ERROR: no xum1541 device found - exit !"
	exit 1
fi
#
echo ""
echo ""
echo ""
if cbmctrl detect | grep 1571
	then
		DRIVETYPE="-d1"	#-d, --drive-type=TYPE		-> specify drive type: 0 or 1541 = 1541 | 1 or 1571 = 1570/1571
	else
		DRIVETYPE="-d0"	#-d, --drive-type=TYPE		-> specify drive type: 0 or 1541 = 1541 | 1 or 1571 = 1570/1571
fi
#
#
#================================================================================================================================================================
#================================================================================================================================================================
#==== Mainloop ==================================================================================================================================================
#================================================================================================================================================================
DISKCOUNTERSTRING=`printf "%.3i\n" $DISKCOUNTERSTART` # change counter number to 3 digits 
cbmctrl reset
cbmctrl status $DEVICENUMBER
echo ""
echo -e "\033[1;32mInsert Disc for creating:  ${DESTINATIONDIR}/${FILEPREFIX}_${DISKCOUNTERSTRING}-${SIDEA}.${FILESUFFIX}\033[0m"
echo -e "\033[1;36mpress RETURN to start or CTRL-C for cancel\033[0m" && read a
while true ; do
	DISKCOUNTERSTRING=`printf "%.3i\n" $DISKCOUNTERSTART` # change counter number to 3 digits 
	PrepareForCopySideA
	$TERMINAL --hide-menubar --hide-toolbar --hide-scrollbar --geometry=50x54 --title="COPY IN PROGRESS" --command="d64copy $XUMADAPTER $DRIVETYPE $STARTTRACK $ENDTRACK $TRANSFERMODE $WARPMODE $RETRYCOUNT $BAMONLY $BAMSAVE $ERRORMAP $TWOSIDED $NOPROGRESS $DEVICENUMBER ${DESTINATIONDIR}/${FILEPREFIX}_${DISKCOUNTERSTRING}-${SIDEA}.${FILESUFFIX}"
	while ps a | grep "d64copy" | grep -v grep >/dev/null ; do sleep 1 ; done
if [ "${TWOSIDED}" = "-2" ] || [ "${TWOSIDED}" = "--two-sided" ]
	then
		sleep 0
	else
		echo ""
		echo -e "\033[1;33mplease insert the same Disk - SideB ... - \033[1;36mCTRL-C for CANCEL\033[0m" && cbmctrl change ${DEVICENUMBER}
		PrepareForCopySideB
		$TERMINAL --hide-menubar --hide-toolbar --hide-scrollbar --geometry=50x54 --title="COPY IN PROGRESS" --command="d64copy $XUMADAPTER $DRIVETYPE $STARTTRACK $ENDTRACK $TRANSFERMODE $WARPMODE $RETRYCOUNT $BAMONLY $BAMSAVE $ERRORMAP $TWOSIDED $NOPROGRESS $DEVICENUMBER ${DESTINATIONDIR}/${FILEPREFIX}_${DISKCOUNTERSTRING}-${SIDEB}.${FILESUFFIX}"
		while ps a | grep "d64copy" | grep -v grep >/dev/null ; do sleep 1 ; done
		echo ""
		echo -e "\033[1;32mplease insert next Disk - SideA ... - \033[1;36mCTRL-C for CANCEL\033[0m" && cbmctrl change ${DEVICENUMBER}
		DISKCOUNTERSTART=$DISKCOUNTERSTART+1
fi
done
#
#
exit

